/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Retrieving Resource Items
    Resource Items are retrieved just as DDOs are.  Since XDOs are are DDOs with extended / 
    extra funcitonality, Sample SItemRetrievalICM may be used for all non-Resource related
    functionality.
    
    Resource Content Access: File, Stream, & Memory
        As shown in SResourceItemCreationICM, Resource Content may be stored directly 
        from a file, from a stream, or loaded into memory and later saved.  The 
        method in which the Resource Content was saved DOES NOT affect how it may
        be retrieved.  The same options are available when accessing the content,
        regardless of how it was originally stored.  A Resource Item added using the 
        "Direct from File" may be accessed later as a Stream or by Local Memory.
    
    Maximizing Efficiency
        Resource Item may be much larger than regular Items, which means that if the 
        Resource Content may not be needed, retrieval of the Item without retrieving the 
        Resource Content is a good solution for faster retrievals overall.  The Resource 
        Content may be retrieved if and when it is specifically needed.  An example of 
        such usage would be in an application displaying a list of hundreds of documents.
        Populating the list does not require resource content.  Only when the user chooses
        to view a particular document, it must then be retrieved.
    
    Retrieval Options
        When actually retrieving the DDO/XDO from the datastore, retrieval options may be 
        specified.  The full set of retrieval options are explained in SItemRetrievalICM.
        The subset relating specifically to Resource Content is shown below.

        Retrieval Option      Constant                     Explanation
        --------------------  ---------------------------  -------------------------------------
        No Resource Content   DK_CM_CONTENT_NO             If it is a Resource Item, do not 
                                                           retrieve the Resource object content.
        ........................................................................................
        Resource Content      DK_CM_CONTENT_YES            If it is a Resource Item, retrieve 
                                                           the Resource object content.
        ........................................................................................
        Only Resource Content DK_CM_CONTENT_ONLY           If it is a Resource Item, retrieve 
                                                           only the Resource object content.
        ----------------------------------------------------------------------------------------
        Note: Constants available in com.ibm.mm.sdk.common.DKConstant

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SResourceItemRetrievalICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Retrieving Resource items, Accessing Resource Content, Maximizing
 *                    Efficiency, Resource-Related Retrieval Options, Validating Files & 
 *                    Streams.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Retrieving Resource Items
 *                    Resource Content Access: File, Stream, & Memory
 *                    Maximizing Efficiency
 *                    Resource-Related Retrieval Options
 *                    Accessing Resource Content
 *                    Accessing As File
 *                    Accessing As Stream
 *                    Validating Files
 *                    Validating Streams
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SResourceItemRetrievalICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                         - SAttributeDefinitionCreationICM
 *                         - SAttributeGroupDefCreationICM
 *                         - SReferenceAttrDefCreationICM
 *                         - SItemTypeCreationICM
 *                    The Item Types must be set to use the specified default Resource Manager.
 *                    Plese run the following samples.
 *                         - SResourceMgrDefSetDefaultICM
 *                         - SSMSCollectionDefSetDefaultICM
 *                    NOTE:  Sample SSampleModelBuildICM pulls together all of the above.  It 
 *                           must be edited first for the RM Confiuration information.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SResourceItemICM_Document1.doc
 *                    SResourceItemICM_Document2.doc
 *                    SResourceItemICM_Text1.txt
 *                    SResourceItemICM_Text2.txt
 *                    SResourceItemICM_Image1.bmp
 *                    SResourceItemICM_Image2.bmp
 ************************************************************************************************/
public class SResourceItemRetrievalICM{

    // Using constants for File Names
    public static final String LOB_FILE1_FILENAME         = "SResourceItemICM_Document1.txt";
    public static final String LOB_FILE2_FILENAME         = "SResourceItemICM_Document2.txt";
    public static final String TEXT_FILE1_FILENAME        = "SResourceItemICM_Text1.txt";
    public static final String TEXT_FILE2_FILENAME        = "SResourceItemICM_Text2.txt";
    public static final String IMAGE_FILE1_FILENAME       = "SResourceItemICM_Picture1.bmp";
    public static final String IMAGE_FILE2_FILENAME       = "SResourceItemICM_Picture2.bmp";
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SResourceItemRetrievalICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SResourceItemRetrievalICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Create Resource Objects for Retrieval Demonstration
            //-------------------------------------------------------------
            // In order to demonstrate retrieval of Resource Items, a few
            // Resource Items must first be created.
            System.out.println("Creating Resource Objects...");

                DKLobICM     lob = (DKLobICM)   dsICM.createDDO("S_lob",   DKConstant.DK_CM_DOCUMENT); // May be any semantic type.
                DKTextICM   text = (DKTextICM)  dsICM.createDDO("S_text",  DKConstant.DK_CM_ITEM);     // Resource Type determined by XDO Classification
                DKImageICM image = (DKImageICM) dsICM.createDDO("S_image", DKConstant.DK_CM_ITEM);     // in the Item Type Definition.

                // Set Mime Type / Type of Content
                lob.setMimeType("application/msword");
                text.setMimeType("text/plain");
                image.setMimeType("image/bmp");
                
                // Store Resource Content & Save Resource Item
                lob.add("SResourceItemICM_Document1.doc");  // Add it to the datastore,
                text.add("SResourceItemICM_Text1.txt");     // storing the content from
                image.add("SResourceItemICM_Picture1.bmp"); // the specified file.

            System.out.println("Created Resource Objects.");

            //-------------------------------------------------------------
            // Recreating XDO objects given PIDs.
            //-------------------------------------------------------------
            // In order to retrieve an Item either the PID Information must
            // be known or Query / Search must be used.  This sample will 
            // use the PID Strings of the items added above.
            System.out.println("Recreating XDO Objects Given PIDs...");
            
                // First we will need to know what items to retrieve.  The PID information is needed.
                String lobPidString   =   lob.getPidObject().pidString();
                String textPidString  =  text.getPidObject().pidString();
                String imagePidString = image.getPidObject().pidString();

                // Knowing Pid strings, let's create new, blank DDOs.
                lob   = (DKLobICM)   dsICM.createDDO(lobPidString);
                text  = (DKTextICM)  dsICM.createDDO(textPidString);
                image = (DKImageICM) dsICM.createDDO(imagePidString);

            System.out.println("Recreated XDO Objects Given PIDs.");

            //-------------------------------------------------------------
            // Retrieving Resource Item, Saving Content Directly to File.
            //-------------------------------------------------------------
            // Regardless of how the Content was originally stored, it may 
            // be written directly to a file when retrieved.
            System.out.println("Retrieving Resource ITem, Saving Content Directly to File...");

                lob.retrieve("myDocument.doc");     // Item will be retrieved and
                text.retrieve("myText.txt");        // resource content will be
                image.retrieve("myPicture.bmp");    // saved in the specified file.

            System.out.println("Retrieved Resource ITem, Saved Content Directly to File.");

            //-------------------------------------------------------------
            // Quick Validation Check Prooving Identical to Original
            //-------------------------------------------------------------
            // Using funcitons provided at the bottom, we will perform a quick
            // validation check to validate and prove that the new files
            // are the exact original specified.
            System.out.println("Validing...");
            
                if(!compareFiles("SResourceItemICM_Document1.doc","myDocument.doc"))
                    throw new Exception("Retrieved Document not identical to original.");
                if(!compareFiles("SResourceItemICM_Text1.txt","myText.txt"))
                    throw new Exception("Retrieved Text File not identical to original.");
                if(!compareFiles("SResourceItemICM_Picture1.bmp","myPicture.bmp"))
                    throw new Exception("Retrieved Picture not identical to original.");

            System.out.println("Valided.");

            //-------------------------------------------------------------
            // Retrieving Resource, Storing Content to Local Memory
            //-------------------------------------------------------------
            // One of the most commonly used methods holds the Resource Content
            // in local memory.  The option below stores it in local memory,
            // alowing easy access to it at a later time.
            System.out.println("Retrieving Resource Item, Storing Content in Local Memory...");

                lob.retrieve(DKConstant.DK_CM_CONTENT_YES);     // Retrieve content
                text.retrieve(DKConstant.DK_CM_CONTENT_YES);    // with object.
                image.retrieve(DKConstant.DK_CM_CONTENT_YES);
                
                // NOTE:  If the content does not need to be retrieved at this point, as
                //        explained in the header documentation, the following can be used:
                // lob.retrieve(DKConstant.DK_CM_CONTENT_NO);

            System.out.println("Retrieved Resource Item, Stored Content in Local Memory.");

            //-------------------------------------------------------------
            // Store content from resource memory to a file.
            //-------------------------------------------------------------
            // Now that the Resource Item has been retrieved, with content in
            // memory, the content may be easily stored to a file.
            System.out.println("Storing Content from Resource Memory to a File...");
    
                lob.getContentToClientFile("myDocument.doc",1);  // "1" means overwrite any existing file.
                text.getContentToClientFile("myText.txt",1);     // Please refer to the Applicationg Programming Reference
                image.getContentToClientFile("myPicture.bmp",1); // (Javadoc) for more information.

            System.out.println("Stored Content from Resource Memory to a File.");

            //-------------------------------------------------------------
            // Make content available through a stream object
            //-------------------------------------------------------------
            System.out.println("Making Content Available Through a Stream Object...");

                InputStream inputStream1 = lob.getInputStream(-1,-1,-1); // -1 is the default.
                InputStream inputStream2 = text.getInputStream(-1,-1,-1); // -1 is the default.
                InputStream inputStream3 = image.getInputStream(-1,-1,-1); // -1 is the default.

            System.out.println("Made Content Available Through a Stream Object.");

            //-------------------------------------------------------------
            // Quick Validation Check Prooving Identical to Original
            //-------------------------------------------------------------
            System.out.println("Validing Streams...");
            
                if(!compareStreamToFile(inputStream1,"SResourceItemICM_Document1.doc"))
                    throw new Exception("Retrieved Document Stream not identical to original.");
                if(!compareStreamToFile(inputStream2,"SResourceItemICM_Text1.txt"))
                    throw new Exception("Retrieved Text Steam not identical to original.");
                if(!compareStreamToFile(inputStream3,"SResourceItemICM_Picture1.bmp"))
                    throw new Exception("Retrieved Picture Stream not identical to original.");

            System.out.println("Valided Streams.");

            //-------------------------------------------------------------
            // Quick Validation for 2nd Set of Files Retrieved.
            //-------------------------------------------------------------
            System.out.println("Validing...");
            
                if(!compareFiles("SResourceItemICM_Document1.doc","myDocument.doc"))
                    throw new Exception("Retrieved Document not identical to original.");
                if(!compareFiles("SResourceItemICM_Text1.txt","myText.txt"))
                    throw new Exception("Retrieved Text File not identical to original.");
                if(!compareFiles("SResourceItemICM_Picture1.bmp","myPicture.bmp"))
                    throw new Exception("Retrieved Picture not identical to original.");

            System.out.println("Valided.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    /**
    * Complare two files, return true if same, false if not.  Performs Binary
    * Comparison.
    * @param file1  One file to compare.
    * @param file2  A second file to compare to the first.
    * @return  Returns true if they are the same, false if not.
    */
    public static boolean compareFiles(String file1, String file2) throws Exception{
        java.io.FileInputStream f1 = new FileInputStream(file1);
        java.io.FileInputStream f2 = new FileInputStream(file2);
        boolean areEqual = true;
        boolean stop = false;
        // read a byte from each until both hit end.
        while(!stop){ // keep going until told to stop.
            int byte1 = f1.read();
            int byte2 = f2.read();
            if( (byte1==-1) && (byte2==-1) ){ // if both reached the end, stop now
                areEqual = true;
                stop = true;
            }
            else if( (byte1==-1) && (byte2!=-1) ){ // if one ends before the other, they are not the same.
                areEqual = false;
                stop = true;
            }
            else if( (byte1!=-1) && (byte2==-1) ){ // if one ends before the other, they are not the same.
                areEqual = false;
                stop = true;
            }
            if(byte1 != byte2){ // if at any point, they differ, return false.
                areEqual = false;
                stop = true;
            }
        }
        f1.close(); // close the file
        f2.close(); // close the file
        return(areEqual);
    }
    /**
    * Complare a file and an input stream, return true if same, false if not.  Performs
    * Binary Comparison.
    * @param inputStream  InputStream to compare.
    * @param file  A file to compare to the input stream.
    * @return  Returns true if they are the same, false if not.
    */
    public static boolean compareStreamToFile(InputStream is, String file) throws Exception{
        java.io.FileInputStream f = new FileInputStream(file);
        boolean areEqual = true;
        boolean stop = false;
        // read a byte from each until both hit end.
        while(!stop){ // keep going until told to stop.
            int byte1 = is.read();
            int byte2 = f.read();
            if( (byte1==-1) && (byte2==-1) ){ // if both reached the end, stop now
                areEqual = true;
                stop = true;
            }
            else if( (byte1==-1) && (byte2!=-1) ){ // if one ends before the other, they are not the same.
                areEqual = false;
                stop = true;
            }
            else if( (byte1!=-1) && (byte2==-1) ){ // if one ends before the other, they are not the same.
                areEqual = false;
                stop = true;
            }
            if(byte1 != byte2){ // if at any point, they differ, return false.
                areEqual = false;
                stop = true;
            }
        }
        is.close(); // close the inputstream.
        f.close(); // close the file
        return(areEqual);
    }

}//end class SResourceItemRetrievalICM
              
